@@ -1,4 +1,4 @@
-use crate::nodes::Expression;
+use crate::nodes::{Expression, Token};
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum BinaryOperator {
@@ -68,7 +68,7 @@ impl BinaryOperator {
         }
     }
 
-    pub fn to_str(&self) -> &str {
+    pub fn to_str(&self) -> &'static str {
         match self {
             Self::And => "and",
             Self::Or => "or",
@@ -111,6 +111,7 @@ pub struct BinaryExpression {
     operator: BinaryOperator,
     left: Expression,
     right: Expression,
+    token: Option<Token>,
 }
 
 impl BinaryExpression {
@@ -123,9 +124,25 @@ impl BinaryExpression {
             operator,
             left: left.into(),
             right: right.into(),
+            token: None,
         }
     }
 
+    pub fn with_token(mut self, token: Token) -> Self {
+        self.token = Some(token);
+        self
+    }
+
+    #[inline]
+    pub fn set_token(&mut self, token: Token) {
+        self.token = Some(token);
+    }
+
+    #[inline]
+    pub fn get_token(&self) -> Option<&Token> {
+        self.token.as_ref()
+    }
+
     #[inline]
     pub fn mutate_left(&mut self) -> &mut Expression {
         &mut self.left
