@@ -71,9 +71,9 @@ impl LuaValue {
     /// Attempt to convert the Lua value into an expression node.
     pub fn to_expression(self) -> Option<Expression> {
         match self {
-            Self::False => Some(Expression::False),
-            Self::True => Some(Expression::True),
-            Self::Nil => Some(Expression::Nil),
+            Self::False => Some(Expression::from(false)),
+            Self::True => Some(Expression::from(true)),
+            Self::Nil => Some(Expression::nil()),
             Self::String(value) => Some(StringExpression::from_value(value).into()),
             Self::Number(value) => Some(Expression::from(value)),
             _ => None,
@@ -105,6 +105,16 @@ impl LuaValue {
         }
         .unwrap_or(self)
     }
+
+    /// Attempt to convert the Lua value into a string value. This will convert numbers when
+    /// possible and return the same value otherwise.
+    pub fn string_coercion(self) -> Self {
+        match &self {
+            Self::Number(value) => Some(Self::String(format!("{}", value))),
+            _ => None,
+        }
+        .unwrap_or(self)
+    }
 }
 
 impl Default for LuaValue {
